/*
 * dcl.c -
 *
 * Author: gotoken
 * $Id: dcl.c,v 1.3 2000/04/27 00:28:44 gotoken Exp $
 *
 * Copyright (C) 2000 GFD Dennou Club. All rights reserved.
 *
 */

#include <math.h>
#include "ruby.h"

VALUE mDCL;
#define STRCPY(buf, str)                                \
    Check_Type(str, T_STRING);                          \
    buf = (char *)malloc(RSTRING(str)->len + 1);        \
    memcpy(buf, RSTRING(str)->ptr, RSTRING(str)->len);  \
    buf[RSTRING(str)->len] = '\0';

#define BE_FLOAT(x)  ((float)RFLOAT(rb_Float(x))->value)
#define BE_INT(x)    (NUM2INT(rb_Integer(x)))

static float *
ARY2CFLOATARY(dest, src)
    float *dest;
    VALUE src;
{
    int i,len;
    VALUE *ptr;

    Check_Type(src, T_ARRAY);
    len = RARRAY(src)->len;
    ptr = RARRAY(src)->ptr;
    dest = (float*)malloc(sizeof(float)*len);
    for(i = 0; i < len; i++) {
	dest[i] = BE_FLOAT(ptr[i]);
    }
    return dest;
}

static VALUE
gropn(self,viws)
    VALUE self, viws;     
{
    int iws;

    iws = NUM2INT(rb_Integer(viws));
    gropn_(&iws);
    return Qnil;
}

static VALUE
grcls(self)
    VALUE self;     
{
    grcls_();
    return Qnil;
}

static VALUE
grfrm(self)
    VALUE self;     
{
    grfrm_();
    return Qnil;
}

static VALUE
sgpwsn(self)
    VALUE self;     
{
    sgpwsn_();
    return Qnil;
}

static VALUE
ussttl(self, xt, xu, yt, yu)
    VALUE self, xt, xu, yt, yu;
{
    char *buf0, *buf1, *buf2, *buf3;

    STRCPY(buf0, xt);
    STRCPY(buf1, xu);
    STRCPY(buf2, yt);
    STRCPY(buf3, yu);
    ussttl_(buf0, buf1, buf2, buf3, 
	    strlen(buf0), strlen(buf1), 
	    strlen(buf2), strlen(buf3));
    free(buf0);
    free(buf1);
    free(buf2);
    free(buf3);
    return Qnil;
}

static VALUE
usgrph(self, n, x, y)
    VALUE self, n, x, y;
{
    int i;
    int m = NUM2INT(n);
    VALUE *tmp0, *tmp1;
    float *xp, *yp;

    Check_Type(x, T_ARRAY);
    Check_Type(y, T_ARRAY);

    if (RARRAY(x)->len < m || RARRAY(y)->len < m) 
	rb_raise(rb_eArgError, "incorrect size of array");
    tmp0 = RARRAY(x)->ptr;
    tmp1 = RARRAY(y)->ptr;

    xp = ALLOCA_N(float, m);
    yp = ALLOCA_N(float, m);

    for (i = 0; i < m; i++) {
	xp[i] = (float)RFLOAT(rb_Float(tmp0[i]))->value;
	yp[i] = (float)RFLOAT(rb_Float(tmp1[i]))->value;
    }
    
    usgrph_(&m, xp, yp);
    return Qnil;
}
/*
  CALL USSPNT( NMAX, X, Y )
  CALL USPFIT
  CALL GRSTRF
  CALL USDAXS
  CALL UULIN( NMAX, X, Y )
*/

static VALUE
usspnt(self, n, x, y)
    VALUE self, n, x, y;
{
    int i;
    int m = NUM2INT(n);
    VALUE *tmp0, *tmp1;
    float *xp, *yp;

    Check_Type(x, T_ARRAY);
    Check_Type(y, T_ARRAY);

    if (RARRAY(x)->len < m || RARRAY(y)->len < m) 
	rb_raise(rb_eArgError, "incorrect size of array");
    tmp0 = RARRAY(x)->ptr;
    tmp1 = RARRAY(y)->ptr;

    xp = ALLOCA_N(float, m);
    yp = ALLOCA_N(float, m);

    for (i = 0; i < m; i++) {
	xp[i] = (float)RFLOAT(rb_Float(tmp0[i]))->value;
	yp[i] = (float)RFLOAT(rb_Float(tmp1[i]))->value;
    }
    
    usspnt_(&m, xp, yp);
    return Qnil;
}

static VALUE
grstrf(self)
    VALUE self;
{
    grstrf_();
    return Qnil;
}

static VALUE
usdaxs(self)
    VALUE self;
{
    usdaxs_();
    return Qnil;
}

static VALUE
uulin(self, n, x, y)
    VALUE self, n, x, y;
{
    int m;
    float *xp, *yp;
    
    m = BE_INT(n);
    ARY2CFLOATARY(xp, x);
    ARY2CFLOATARY(yp, y);
    uulin_(&m, &xp, &yp);
    free(xp);
    free(yp);
    return Qnil;
}


static VALUE
uulinz(self, n, x, y, t, b)
    VALUE self, n, x, y, t, b;
{
    int i;
    int m = NUM2INT(n);
    int type = NUM2INT(t);
    int bold = NUM2INT(b);
    VALUE *tmp0, *tmp1;
    float *xp, *yp;

    Check_Type(x, T_ARRAY);
    Check_Type(y, T_ARRAY);

    if (RARRAY(x)->len < m || RARRAY(y)->len < m) 
	rb_raise(rb_eArgError, "incorrect size of array");
    tmp0 = RARRAY(x)->ptr;
    tmp1 = RARRAY(y)->ptr;

    xp = ALLOCA_N(float, m);
    yp = ALLOCA_N(float, m);

    for (i = 0; i < m; i++) {
	xp[i] = (float)RFLOAT(rb_Float(tmp0[i]))->value;
	yp[i] = (float)RFLOAT(rb_Float(tmp1[i]))->value;
    }
    
    uulinz_(&m, xp, yp, type, bold);
    return Qnil;
}

static VALUE
sgopn(self, n)
    VALUE self, n;
{
    int m = NUM2INT(rb_Integer(n));

    sgopn_(&m);
    return Qnil;
}

static VALUE
sgcls(self)
    VALUE self;
{
    sgcls_();
    return Qnil;
}

static VALUE
sgfrm(self)
    VALUE self;
{
    sgfrm_();
    return Qnil;
}

static VALUE
slpvpr(self, n)
    VALUE self, n;
{
    int m = NUM2INT(n);

    slpvpr_(&m);
    return Qnil;
}

static VALUE
slpwwr(self, n)
    VALUE self, n;
{
    int m = NUM2INT(n);

    slpwwr_(&m);
    return Qnil;
}

static VALUE
slpwvr(self, n)
    VALUE self, n;
{
    int m = NUM2INT(n);

    slpwvr_(&m);
    return Qnil;
}

static VALUE
slpvpc(self, n, x)
    VALUE self, n, x;
{
    int m = NUM2INT(n);
    float y = RFLOAT(rb_Float(x))->value;

    slpvpc_(&m, y);
    return Qnil;
}

static VALUE
slpwwc(self, n, x)
    VALUE self, n, x;
{
    int m = NUM2INT(n);
    float y = RFLOAT(rb_Float(x))->value;

    slpwwc_(&m, y);
    return Qnil;
}

static VALUE
slpwvc(self, n, x)
    VALUE self, n, x;
{
    int m = NUM2INT(n);
    float y = RFLOAT(rb_Float(x))->value;

    slpwvc_(&m, x);
    return Qnil;
}

static VALUE
scsobj(self, x, y, z)
    VALUE self, x, y, z;
{
    float p0, p1, p2;
    p0 = RFLOAT(rb_Float(x))->value;
    p1 = RFLOAT(rb_Float(y))->value;
    p2 = RFLOAT(rb_Float(z))->value;

    scsobj_(&p0, &p1, &p2);
    return Qnil;
}

static VALUE
scseye(self, x, y, z)
    VALUE self, x, y, z;
{
    float p0, p1, p2;

    p0 = RFLOAT(rb_Float(x))->value;
    p1 = RFLOAT(rb_Float(y))->value;
    p2 = RFLOAT(rb_Float(z))->value;
    scseye_(&p0, &p1, &p2);
    return Qnil;
}

static VALUE
scsprj(self)
    VALUE self;
{
    scsprj_();
    return Qnil;
}

static VALUE
sgrset(self, var, val)
    VALUE self, var, val;
{
    int name;
    float v;
    char *buf;

    v = BE_FLOAT(val);
    STRCPY(buf, var);
    sgiset_(buf, &v, strlen(buf));
    free(buf);
    return Qnil;
} 

static VALUE
sgiset(self, var, val)
    VALUE self, var, val;
{
    int name;
    int v;
    char *buf;

    v = BE_INT(val);
    STRCPY(buf, var);
    sgiset_(buf, &v, strlen(buf));
    free(buf);
    return Qnil;
} 

static VALUE
sglqnp(self)
{
    int n;
    sglqnp_(&n);
    return INT2NUM(n);
}

static VALUE
sglset(self, var, val)
    VALUE self, var, val;
{
    char *buf;
    int v;

    if (val == Qnil || val == Qfalse)
	v = 0;
    else
	v = 1;
    STRCPY(buf, var);
    sglset_(buf, &v, strlen(buf));
    free(buf);
    return Qnil;
}

static VALUE
szpll3(self, x, y, z)
    VALUE self, x, y, z;
{
    float s,t,u;

    s = RFLOAT(rb_Float(x))->value;
    t = RFLOAT(rb_Float(y))->value;
    u = RFLOAT(rb_Float(z))->value;
    szpll3_(&s,&t,&u);
    return Qnil;
}

static VALUE
szcll3(self)
    VALUE self;
{
    szcll3_();
    return Qnil;
}

static VALUE
szmvl3(self, x, y, z)
    VALUE self, x, y, z;
{
    float s,t,u;

    s = RFLOAT(rb_Float(x))->value;
    t = RFLOAT(rb_Float(y))->value;
    u = RFLOAT(rb_Float(z))->value;
    szmvl3_(&s,&t,&u);
    return Qnil;
}

static VALUE
szopl3(self)
{
    szopl3_();
    return Qnil;
}

static VALUE
slmgn(self, a1, a2, a3, a4)
    VALUE self, a1, a2, a3, a4;
{
    float m1, m2, m3, m4;

    m1 = RFLOAT(rb_Float(a1))->value;
    m2 = RFLOAT(rb_Float(a2))->value;
    m3 = RFLOAT(rb_Float(a3))->value;
    m4 = RFLOAT(rb_Float(a4))->value;
    slmgn_(&m1, &m2, &m3, &m4);
    return Qnil;
}

static VALUE
slrat(self, x, y)
    VALUE self, x, y;
{    
    float s, t;
    s = RFLOAT(rb_Float(x))->value;
    t = RFLOAT(rb_Float(y))->value;
    slrat_(&s, &t);
    return Qnil;
}

static VALUE
sgstxi(self, n)
    VALUE self, n;
{
    int m;

    m = NUM2INT(rb_Integer(n));
    sgstxi_(&m);
    return Qnil;
}

static VALUE
sgstxs(self, x)
    VALUE self, x;
{
    float s;

    s = RFLOAT(rb_Float(x))->value;
    sgstxs_(&s);
    return Qnil;
}

static VALUE
sgstxv(self, x, y, n)
    VALUE self, x, y, n;
{
    float s, t;
    int m;

    s = RFLOAT(rb_Float(x))->value;
    t = RFLOAT(rb_Float(y))->value;
    m = NUM2INT(rb_Integer(n));
    sgstxv_(&s, &t, &m);
    return Qnil;
}

static VALUE
sldiv(self, n, x, y)
    VALUE self, n, x, y;
{
    int m, s, t;

    m = NUM2INT(rb_Integer(n));
    s = NUM2INT(rb_Integer(x));
    t = NUM2INT(rb_Integer(y));

    sldiv_(&m, &s, &t);
    return Qnil;
}

static VALUE
sgtxv(self, x, y, str)
    VALUE self, x, y, str;
{
    float s,t;
    char *buf;
    
    s = BE_FLOAT(x);
    t = BE_FLOAT(y);
    STRCPY(buf, str);
    sgtxv_(&s, &t, buf, strlen(buf));
    free(buf);
    return Qnil;
}

static VALUE
slsttl(self, str, a, b, c, d, e)
    VALUE self, str, a, b, c, d, e;
{
    char *buf;
    int n, m;
    float x, y, z;

    x = BE_FLOAT(b);
    y = BE_FLOAT(c);
    z = BE_FLOAT(d);

    n = BE_INT(a);
    m = BE_INT(e);

    STRCPY(buf, str);
    slsttl_(buf, &n, &x, &y, &z, &m, strlen(buf));
    free(buf);
    return Qnil;
}


static VALUE
glrget(self, str)
    VALUE self;
{
    char *buf;
    float x;

    STRCPY(buf, str);
    glrget_(buf, &x, strlen(buf));
    free(buf);
    return rb_float_new(x);
}

static VALUE
grfig(self)
    VALUE self;
{
    grfig_();
    return Qnil;
}

static VALUE
grstrn(self, n)
    VALUE self;
{
    int m = BE_INT(n);
    grstrn_(&m);
    return Qnil;
}

static VALUE
grsvpt(self, r0, r1, r2, r3)
    VALUE self, r0, r1, r2, r3;
{
    float x0, x1, x2, x3;

    x0 = BE_FLOAT(r0);
    x1 = BE_FLOAT(r1);
    x2 = BE_FLOAT(r2);
    x3 = BE_FLOAT(r3);
    grsvpt_(&x0, &x1, &x2, &x3);
    return Qnil;
}

static VALUE
grswnd(self, r0, r1, r2, r3)
    VALUE self, r0, r1, r2, r3;
{
    float x0, x1, x2, x3;

    x0 = BE_FLOAT(r0);
    x1 = BE_FLOAT(r1);
    x2 = BE_FLOAT(r2);
    x3 = BE_FLOAT(r3);
    grswnd_(&x0, &x1, &x2, &x3);
    return Qnil;
}

static VALUE
udcntr(self, r0, i0, i1, i2, r1)
    VALUE self, r0, i0, i1, i2, r1;
{
    int n0, n1, n2;
    float x0, x1;
    
    n0 = BE_INT(i0);
    n1 = BE_INT(i1);
    n2 = BE_INT(i2);
    x0 = BE_FLOAT(r0);
    x1 = BE_FLOAT(r1);
    udcntr_(&x0, &n0, &n1, &n2, &x1);
    return Qnil;
}

static VALUE
uestlv(self, r0, r1, i0)
    VALUE self, r0, r1, i0;
{
    int n0;
    float x0, x1;
    
    n0 = BE_INT(i0);
    x0 = BE_FLOAT(r0);
    x1 = BE_FLOAT(r1);
    uestlv_(&n0, &x0, &x1);
    return Qnil;
}

static VALUE
uetone(self, r0, i0, i1, i2)
    VALUE self, r0, i0, i1, i2;
{
    int n0, n1, n2;
    float x0;
    
    n0 = BE_INT(i0);
    n1 = BE_INT(i1);
    n2 = BE_INT(i2);
    x0 = BE_FLOAT(r0);
    uetone_(&x0, &n0, &n1, &n2);
    return Qnil;
}

static VALUE
ugvect(self, r0, i0, r1, i1, i2, i3)
    VALUE self;
{
    int n0, n1, n2, n3;
    float x0, x1;

    n0 = BE_INT(i0);
    n1 = BE_INT(i1);
    n2 = BE_INT(i2);
    n3 = BE_INT(i3);
    x0 = BE_FLOAT(r0);
    x1 = BE_FLOAT(r1);
    ugvect_(&x0, &n0, &x1, &n1, &n2, &n3);
    return Qnil;
}

static VALUE
uspfit(self)
    VALUE self;
{
    uspfit_();
    return Qnil;
}

static VALUE
uumrk(self, i0, a0, a1, i1, i2, r0)
    VALUE self, i0, a0, a1, i1, i2, r0;
{
    float x0;
    int n0, n1, n2;
    float *f0, *f1;

    n0 = BE_INT(i0);
    n1 = BE_INT(i1);
    n2 = BE_INT(i2);
    x0 = BE_FLOAT(r0);
    ARY2CFLOATARY(f0, a0);
    ARY2CFLOATARY(f1, a1);
    uumrk_(&n0, &f0, &f1, &n1, &n2, &x0);
    free(f0);
    free(f1);
    return Qnil;
}

static VALUE
uuslni(self, i0)
    VALUE self, i0;
{
    int n0;
    
    n0 = BE_INT(i0);
    uuslni_(&n0);
    return Qnil;
}

static VALUE
uuslnt(self, i0)
    VALUE self, i0;
{
    int n0;
    
    n0 = BE_INT(i0);
    uuslnt_(&n0);
    return Qnil;
}

static VALUE
uusmki(self, i0)
    VALUE self, i0;
{
    int n0;
    
    n0 = BE_INT(i0);
    uusmki_(&n0);
    return Qnil;
}

static VALUE
uusmks(self, i0)
    VALUE self, i0;
{
    int n0;

    n0 = BE_INT(i0);
    uusmks_(&n0);
    return Qnil;
}

static VALUE
uusmkt(self, r0)
    VALUE self, r0;
{
    float x0;

    x0 = BE_FLOAT(r0);
    uusmkt_(&x0);
    return Qnil;
}


static VALUE
scspln(self, i0, i1, i2)
    VALUE self, i0, i1, i2;
{
    int   n0, n1, n2;

    n0 = BE_INT(i0);
    n1 = BE_INT(i1);
    n2 = BE_INT(i2);
    scspln_(&n0, &n1, &n2);
    return Qnil;
}

static VALUE
sgsmpl(self, r0, r1, r2, r3)
    VALUE self, r0, r1, r2, r3;
{
    float x0, x1, x2, x3;

    x0 = BE_FLOAT(r0);
    x1 = BE_FLOAT(r1);
    x2 = BE_FLOAT(r2);
    x3 = BE_FLOAT(r3);

    sgsmpl_(&x0, &x1, &x2, &x3);
    return Qnil;
}

static VALUE
sgssim(self, r0, r1, r2, r3)
    VALUE self, r0, r1, r2, r3;
{
    float x0, x1, x2, x3;

    x0 = BE_FLOAT(r0);
    x1 = BE_FLOAT(r1);
    x2 = BE_FLOAT(r2);
    x3 = BE_FLOAT(r3);

    sgssim_(&x0, &x1, &x2, &x3);
    return Qnil;
}

static VALUE
sgstrf(self)
    VALUE self;
{
    sgstrf_();
    return Qnil;
}

static VALUE
sgstrn(self, i0)
    VALUE self, i0;
{
    int   n0;

    n0 = BE_INT(i0);
    sgstrn_(&n0);
    return Qnil;
}

static VALUE
sgstxy(self, r0, r1, r2, r3)
    VALUE self, r0, r1, r2, r3;
{
    float x0, x1, x2, x3;

    x0 = BE_FLOAT(r0);
    x1 = BE_FLOAT(r1);
    x2 = BE_FLOAT(r2);
    x3 = BE_FLOAT(r3);
    sgstxy_(&r0, &r1, &r2, &r3);
    return Qnil;
}

static VALUE
sgsvpt(self, r0, r1, r2, r3, r4, r5)
    VALUE self, r0, r1, r2, r3, r4, r5;
{
    float x0, x1, x2, x3, x4, x5;

    x0 = BE_FLOAT(r0);
    x1 = BE_FLOAT(r1);
    x2 = BE_FLOAT(r2);
    x3 = BE_FLOAT(r3);
    x4 = BE_FLOAT(r4);
    x5 = BE_FLOAT(r5);

    sgsvpt_(&x0, &x1, &x2, &x3, &x4, &x5);
    return Qnil;
}

static VALUE
sgswnd(self, r0, r1, r2, r3)
    VALUE self, r0, r1, r2, r3;
{
    float x0, x1, x2, x3;

    x0 = BE_FLOAT(r0);
    x1 = BE_FLOAT(r1);
    x2 = BE_FLOAT(r2);
    x3 = BE_FLOAT(r3);
    sgswnd_(&x0, &x1, &x2, &x3);
    return Qnil;
}

static VALUE
sgtxzv(self, r0, r1, s0, r2, i0, i1, i2)
    VALUE self;
{
    float x0, x1, x2;
    int   n0, n1, n2;
    char *b0;

    x0 = BE_FLOAT(r0);
    x1 = BE_FLOAT(r1);
    x2 = BE_FLOAT(r2);
    n0 = BE_INT(i0);
    n1 = BE_INT(i1);
    n2 = BE_INT(i2);
    STRCPY(b0, s0);
    sgtxzv_(&x0, &x1, b0, &x2, &n0, &n1, &n2, strlen(b0));
    free(b0);
    return Qnil;
}

static VALUE
umiset(self, s0, i0)
    VALUE self, s0, i0;
{
    int   n0;
    char *b0;
    n0 = BE_INT(i0);
    STRCPY(b0, s0);
    umiset_(b0, &n0, strlen(b0));
    return Qnil;
}

static VALUE
umlset(self, s0, val)
    VALUE self, s0, val;
{
    char *b0;
    int v;

    if (val == Qnil || val == Qfalse)
	v = 0;
    else
	v = 1;
    STRCPY(b0, s0);
    umlset_(b0, &v, strlen(b0));
    free(b0);
    return Qnil;
}

static VALUE
umpglb(self)
    VALUE self;
{
    umpglb_();
    return Qnil;
}

static VALUE
umpmap(self, s0)
    VALUE self, s0;
{
    char *b0;

    STRCPY(b0, s0);
    umpmap_(b0, strlen(b0));
    free(b0);
    return Qnil;
}

static VALUE
umrset(self, s0, r0)
    VALUE self, s0, r0;
{
    float x0;
    char *b0;

    STRCPY(b0, s0);
    x0 = BE_FLOAT(r0);
    umrset_(b0, &x0, strlen(b0));
    free(b0);
    return Qnil;
}

void
Init_dcl()
{
    mDCL = rb_define_module("DCL");
    rb_define_module_function(mDCL, "gropn", gropn, 1);
    rb_define_module_function(mDCL, "grcls", grcls, 0);
    rb_define_module_function(mDCL, "grfrm", grfrm, 0);
    rb_define_module_function(mDCL, "sgpwsn", sgpwsn, 0);
    rb_define_module_function(mDCL, "ussttl", ussttl, 4);
    rb_define_module_function(mDCL, "usgrph", usgrph, 3);
    rb_define_module_function(mDCL, "usspnt", usspnt, 3);
    rb_define_module_function(mDCL, "uspfit", uspfit, 0);
    rb_define_module_function(mDCL, "grstrf", grstrf, 0);
    rb_define_module_function(mDCL, "usdaxs", usdaxs, 0);
    rb_define_module_function(mDCL, "uulin", uulin, 3);
    rb_define_module_function(mDCL, "uulinz", uulinz, 5);

    rb_define_module_function(mDCL, "sgopn", sgopn, 1);
    rb_define_module_function(mDCL, "sgcls", sgcls, 0);
    rb_define_module_function(mDCL, "sgfrm", sgfrm, 0);
    rb_define_module_function(mDCL, "slpvpr", slpvpr, 1);
    rb_define_module_function(mDCL, "slpwwr", slpwwr, 1);
    rb_define_module_function(mDCL, "slpwvr", slpwvr, 1);
    rb_define_module_function(mDCL, "slpvpc", slpvpc, 1);
    rb_define_module_function(mDCL, "slpwwc", slpwwc, 1);
    rb_define_module_function(mDCL, "slpwvc", slpwvc, 1);
    rb_define_module_function(mDCL, "scsobj", scsobj, 3);
    rb_define_module_function(mDCL, "scseye", scseye, 3);
    rb_define_module_function(mDCL, "scsprj", scsprj, 0);
    rb_define_module_function(mDCL, "sgrset", sgrset, 2);
    rb_define_module_function(mDCL, "sgiset", sgiset, 2);
    rb_define_module_function(mDCL, "sglset", sglset, 2);

    rb_define_module_function(mDCL, "szopl3", szopl3, 0);
    rb_define_module_function(mDCL, "szmvl3", szmvl3, 3);
    rb_define_module_function(mDCL, "szpll3", szpll3, 3);
    rb_define_module_function(mDCL, "szcll3", szcll3, 0);

    rb_define_module_function(mDCL, "slmgn", slmgn, 4);
    rb_define_module_function(mDCL, "sldiv", sldiv, 3);
    rb_define_module_function(mDCL, "slrat", slrat, 2);
    rb_define_module_function(mDCL, "sgstxi", sgstxi, 1);
    rb_define_module_function(mDCL, "sgstxs", sgstxs, 1);
    rb_define_module_function(mDCL, "sgstxv", sgstxv, 3);
    rb_define_module_function(mDCL, "sgtxv", sgtxv, 3);
    rb_define_module_function(mDCL, "slsttl", slsttl, 6);
    rb_define_module_function(mDCL, "sgtxzv", sgtxzv, 7);
    rb_define_module_function(mDCL, "sglqnp", sglqnp, 0);

    rb_define_module_function(mDCL, "glrget", glrget, 1);
    rb_define_module_function(mDCL, "grfig", grfig, 0);
    rb_define_module_function(mDCL, "grstrn", grstrn, 1);
    rb_define_module_function(mDCL, "grsvpt", grsvpt, 4);
    rb_define_module_function(mDCL, "grswnd", grswnd, 4);
    rb_define_module_function(mDCL, "udcntr", udcntr, 5);
    rb_define_module_function(mDCL, "uestlv", uestlv, 3);
    rb_define_module_function(mDCL, "uetone", uetone, 4);
    rb_define_module_function(mDCL, "ugvect", ugvect, 6);
    rb_define_module_function(mDCL, "uumrk", uumrk, 6);
    rb_define_module_function(mDCL, "uuslni", uuslni, 1);
    rb_define_module_function(mDCL, "uuslnt", uuslnt, 1);
    rb_define_module_function(mDCL, "uusmki", uusmki, 1);
    rb_define_module_function(mDCL, "uusmks", uusmks, 1);
    rb_define_module_function(mDCL, "uusmkt", uusmkt, 1);

    rb_define_module_function(mDCL, "scspln", scspln, 3);
    rb_define_module_function(mDCL, "sgsmpl", sgsmpl, 3);
    rb_define_module_function(mDCL, "sgssim", sgssim, 3);
    rb_define_module_function(mDCL, "sgstrf", sgstrf, 0);
    rb_define_module_function(mDCL, "sgstrn", sgstrn, 1);
    rb_define_module_function(mDCL, "sgstxy", sgstxy, 4);
    rb_define_module_function(mDCL, "sgsvpt", sgsvpt, 4);
    rb_define_module_function(mDCL, "sgswnd", sgswnd, 4);
    rb_define_module_function(mDCL, "umiset", umiset, 2);
    rb_define_module_function(mDCL, "umlset", umlset, 2);
    rb_define_module_function(mDCL, "umpglb", umpglb, 0);
    rb_define_module_function(mDCL, "umpmap", umpmap, 1);
    rb_define_module_function(mDCL, "umrset", umrset, 2);
}
